import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;


enum Lakresz {
    FUVES_RESZ,
    HOMOK,
    MEDENCE,
    MADARHAZ,
    HULLOHAZ
}

abstract class Allat {
    private String megnevezes;
    private int eletkor;
    private double suly;
    private Lakresz lakresz;

    public Allat(String megnevezes, int eletkor, double suly, Lakresz lakresz) {
        this.megnevezes = megnevezes;
        this.eletkor = eletkor;
        this.suly = suly;
        this.lakresz = lakresz;
    }

    public String getMegnevezes() {
        return megnevezes;
    }

    public void setMegnevezes(String megnevezes) {
        this.megnevezes = megnevezes;
    }

    public int getEletkor() {
        return eletkor;
    }

    public void setEletkor(int eletkor) {
        this.eletkor = eletkor;
    }

    public double getSuly() {
        return suly;
    }

    public void setSuly(double suly) {
        this.suly = suly;
    }

    public Lakresz getLakresz() {
        return lakresz;
    }

    public void setLakresz(Lakresz lakresz) {
        this.lakresz = lakresz;
    }

    @Override
    public String toString() {
        return "megnevezés: " + megnevezes + ", életkor: " + eletkor + " év, súly: " + suly + " kg, lakrész: " + lakresz;
    }
}

class Emlos extends Allat {
    private String eletter;

    public Emlos(String megnevezes, int eletkor, double suly, Lakresz lakresz, String eletter) {
        super(megnevezes, eletkor, suly, lakresz);
        this.eletter = eletter;
    }

    public String getEletter() {
        return eletter;
    }

    public void setEletter(String eletter) {
        this.eletter = eletter;
    }

    @Override
    public String toString() {
        return super.toString() + ", Élettér: " + eletter;
    }
}

class Madar extends Allat {
    private int szarnyfesztav;

    public Madar(String megnevezes, int eletkor, double suly, Lakresz lakresz, int szarnyfesztav) {
        super(megnevezes, eletkor, suly, lakresz);
        this.szarnyfesztav = szarnyfesztav;
    }

    public int getSzarnyfesztav() {
        return szarnyfesztav;
    }

    public void setSzarnyfesztav(int szarnyfesztav) {
        this.szarnyfesztav = szarnyfesztav;
    }

    @Override
    public String toString() {
        return super.toString() + ", szárnyfesztáv: " + szarnyfesztav + " cm";
    }
}

class Hullo extends Allat {
    private boolean pikkelyes;

    public Hullo(String megnevezes, int eletkor, double suly, Lakresz lakresz, boolean pikkelyes) {
        super(megnevezes, eletkor, suly, lakresz);
        this.pikkelyes = pikkelyes;
    }

    public boolean isPikkelyes() {
        return pikkelyes;
    }

    public void setPikkelyes(boolean pikkelyes) {
        this.pikkelyes = pikkelyes;
    }

    @Override
    public String toString() {
        return super.toString() + ", Pikkelyes: " + (pikkelyes ? "igen" : "nem");
    }
}

class Rovar extends Allat {
    private int labakSzama;

    public Rovar(String megnevezes, int eletkor, double suly, Lakresz lakresz, int labakSzama) {
        super(megnevezes, eletkor, Math.min(suly, 0.5), lakresz);
        this.labakSzama = labakSzama;
        if (suly > 0.5) {
            System.out.println("Figyelmeztetés: A rovar súlya túl nagy, ezért 0.01 kg alap értékre lett állítva.");
            setSuly(0.01);
        }
    }

    public int getLabakSzama() {
        return labakSzama;
    }

    public void setLabakSzama(int labakSzama) {
        this.labakSzama = labakSzama;
    }

    @Override
    public String toString() {
        return super.toString() + ", Lábak száma: " + labakSzama;
    }
}

class Allatkert {
    private String nev;
    private String cim;
    private String telefonszam;
    private String email;
    private List<Allat> allatok;

    public Allatkert(String nev, String cim, String telefonszam, String email) {
        this.nev = nev;
        this.cim = cim;
        this.telefonszam = telefonszam;
        this.email = email;
        this.allatok = new ArrayList<>();
    }

    public void hozzaadAllat(Allat allat) {
        allatok.add(allat);
        System.out.println("Új állat hozzáadva: " + allat.getMegnevezes());
    }

    public void torolAllat(String megnevezes) {
        allatok.removeIf(allat -> allat.getMegnevezes().equals(megnevezes));
        System.out.println("Állat törölve: " + megnevezes);
    }

    public String keres(String allatNeve) {
        for (Allat allat : allatok) {
            if (allat.getMegnevezes().equalsIgnoreCase(allatNeve)) {
                return allat.toString();
            }
        }
        return "Nincs ilyen nevű állat.";
    }

    public void listazAllatokat() {
        System.out.println("\n\t[+2] Az állatok típusonkénti kilistázása súly szerint csökkenő sorrendben'");
        allatok.stream()
                .collect(Collectors.groupingBy(Allat::getClass))
                .forEach((tipus, lista) -> {
                    System.out.println("\nTípus: " + tipus.getSimpleName());
                    List<Allat> rendezettLista = lista.stream()
                            .sorted(Comparator.comparingDouble(Allat::getSuly).reversed())
                            .collect(Collectors.toList());
                    for (int i = 0; i < rendezettLista.size(); i++) {
                        Allat allat = rendezettLista.get(i);
                        System.out.println((i + 1) + ". " + allat);
                    }
                });
    }   

    
    public int getAdottTipusuAllatokSzama(Class<?> tipus) {
        List<Allat> adottTipusuAllatok = allatok.stream()
                .filter(tipus::isInstance)
                .collect(Collectors.toList());
        
        System.out.println("Típus: " + tipus.getSimpleName());
        for (int i = 0; i < adottTipusuAllatok.size(); i++) {
            System.out.println((i + 1) + ". " + adottTipusuAllatok.get(i));
        }
    
        return adottTipusuAllatok.size();
    }

    public void irjFajlba(String fileNev) {
        System.out.println("\n\t[+3] Minden olyan állatnév kiírása fájlba, ami legalább 4 éves, és a nevébe van 'a' betű.\n");
        try (FileWriter writer = new FileWriter(fileNev)) {
            for (Allat allat : allatok) {
                if (allat.getMegnevezes().contains("a") && allat.getEletkor() >= 4) {
                    writer.write(allat + "\n");
                }
            }
            System.out.println("Fájlba írás kész: " + fileNev);
        } catch (IOException e) {
            System.err.println("Hiba a fájl írása közben: " + e.getMessage());
        }
    }
}

public class Main {
    public static void main(String[] args) {
        Allatkert allatkert = new Allatkert("Fővárosi Állatkert", "Budapest, Állatkerti krt. 6-12.", "123-4567", "info@allatkert.hu");

        // állatok betöltése
        System.out.println("\n\t[+0] Állatok hozzáadása az állatkerthez\n");
        allatkert.hozzaadAllat(new Emlos("Oroszlán", 5, 190.5, Lakresz.FUVES_RESZ, "Szavanna"));
        allatkert.hozzaadAllat(new Emlos("Gorilla", 9, 100.5, Lakresz.FUVES_RESZ, "Szavanna"));
        allatkert.hozzaadAllat(new Emlos("Majom", 8, 90.5, Lakresz.FUVES_RESZ, "Szavanna"));
        allatkert.hozzaadAllat(new Emlos("Gepárd", 6, 80.5, Lakresz.FUVES_RESZ, "Szavanna"));
        allatkert.hozzaadAllat(new Emlos("Tigris", 4, 210.0, Lakresz.FUVES_RESZ, "Dzsungel"));
        allatkert.hozzaadAllat(new Emlos("Elefánt", 15, 5400.0, Lakresz.FUVES_RESZ, "Szavanna"));
        allatkert.hozzaadAllat(new Emlos("Zebra", 7, 300.0, Lakresz.FUVES_RESZ, "Szavanna"));
        allatkert.hozzaadAllat(new Emlos("Hiéna", 6, 70.0, Lakresz.FUVES_RESZ, "Szavanna"));
        allatkert.hozzaadAllat(new Emlos("Panda", 12, 120.0, Lakresz.FUVES_RESZ, "Bambuszerdő"));
        allatkert.hozzaadAllat(new Emlos("Farkas", 8, 80.0, Lakresz.FUVES_RESZ, "Erdő"));
        
        allatkert.hozzaadAllat(new Madar("Sas", 3, 4.0, Lakresz.MADARHAZ, 220));
        allatkert.hozzaadAllat(new Madar("Papagáj", 2, 0.9, Lakresz.MADARHAZ, 50));
        allatkert.hozzaadAllat(new Madar("Bagoly", 5, 2.3, Lakresz.MADARHAZ, 120));
        allatkert.hozzaadAllat(new Madar("Flamingó", 6, 3.5, Lakresz.MADARHAZ, 160));
        allatkert.hozzaadAllat(new Madar("Varjú", 4, 1.1, Lakresz.MADARHAZ, 80));
        allatkert.hozzaadAllat(new Madar("Galamb", 2, 0.8, Lakresz.MADARHAZ, 40));
        allatkert.hozzaadAllat(new Madar("Fecske", 1, 0.3, Lakresz.MADARHAZ, 35));
        allatkert.hozzaadAllat(new Madar("Kakadu", 6, 1.0, Lakresz.MADARHAZ, 60));

        allatkert.hozzaadAllat(new Hullo("Krokodil", 12, 400.0, Lakresz.MEDENCE, true));
        allatkert.hozzaadAllat(new Hullo("Krokodil", 15, 450.0, Lakresz.MEDENCE, true));
        allatkert.hozzaadAllat(new Hullo("Krokodil", 10, 380.0, Lakresz.MEDENCE, true));
        allatkert.hozzaadAllat(new Hullo("Krokodil", 8, 360.0, Lakresz.MEDENCE, true));
        allatkert.hozzaadAllat(new Hullo("Gyík", 1, 0.2, Lakresz.FUVES_RESZ, false));
        allatkert.hozzaadAllat(new Hullo("Teknős", 100, 200.0, Lakresz.MEDENCE, false));
        allatkert.hozzaadAllat(new Hullo("Kaméleon", 3, 0.5, Lakresz.FUVES_RESZ, false));
        allatkert.hozzaadAllat(new Hullo("Boa", 7, 20.0, Lakresz.MEDENCE, true));
        allatkert.hozzaadAllat(new Hullo("Viper", 4, 1.5, Lakresz.FUVES_RESZ, true));
        allatkert.hozzaadAllat(new Hullo("Alligátor", 11, 350.0, Lakresz.MEDENCE, true));
        allatkert.hozzaadAllat(new Hullo("Iguána", 5, 3.0, Lakresz.FUVES_RESZ, false));
        allatkert.hozzaadAllat(new Hullo("Kígyó", 9, 25.0, Lakresz.MEDENCE, true));

        allatkert.hozzaadAllat(new Rovar("Nagy bogár", 1, 0.6, Lakresz.FUVES_RESZ, 6));
        allatkert.hozzaadAllat(new Rovar("Hangya", 1, 0.01, Lakresz.FUVES_RESZ, 6));
        allatkert.hozzaadAllat(new Rovar("Méh", 1, 0.03, Lakresz.FUVES_RESZ, 6));
        allatkert.hozzaadAllat(new Rovar("Szöcske", 2, 0.05, Lakresz.FUVES_RESZ, 6));
        allatkert.hozzaadAllat(new Rovar("Katicabogár", 1, 0.02, Lakresz.FUVES_RESZ, 6));
        allatkert.hozzaadAllat(new Rovar("Pillangó", 1, 0.04, Lakresz.FUVES_RESZ, 6));
        allatkert.hozzaadAllat(new Rovar("Tücsök", 3, 0.07, Lakresz.FUVES_RESZ, 6));
        
        //allat torlesere minta:
        System.out.println("\n\t[+0] Állat törlése\n");
        allatkert.hozzaadAllat(new Emlos("Vak egér", 8, 1.0, Lakresz.FUVES_RESZ, "Erdő"));
        allatkert.torolAllat("Vak egér");

        // kikeresés
        System.out.println("\n\t[+0] Állat kikeresése név alapján\n");
        System.out.println(allatkert.keres("Oroszlán"));

        // állatok száma, és darab szerint kilistázgatása
        System.out.println("\n\t[+0] Típusonként kilistázott állatok\n");
        System.out.println("Emlősök száma: " + allatkert.getAdottTipusuAllatokSzama(Emlos.class) + "\n");
        System.out.println("Madarak száma: " + allatkert.getAdottTipusuAllatokSzama(Madar.class) + "\n");
        System.out.println("Hüllők száma: " + allatkert.getAdottTipusuAllatokSzama(Hullo.class) + "\n");
        System.out.println("Rovarok száma: " + allatkert.getAdottTipusuAllatokSzama(Rovar.class));

        //+1 es feladat megoldva a 140 - 143 as sorba

        // +2 kilistázás súly szerint
        allatkert.listazAllatokat();

        // +3 fájlbaírás
        allatkert.irjFajlba("idosebb_allatok.txt");
    }
}
